<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class AssociacaoFinanceiraController extends Crud {

    protected $dataSourceName = 'fpa_incideparcpagar_get';
    protected $dataSourceName_Set = 'fpa_incideparcpagar_set';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager); 
    }

    public function getAssociacaoFinanceira (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria(); 
        
        $conditions = [];
        foreach ($filterData->getConditions() as $condition) {
            if ($condition['columnName'] !== 'NRINSCRICAOPARC') {
                $conditions[] = $condition;
            }
        }
        $filterData = new FilterCriteria(
            $this->dataSourceName,
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );


        $newFilterData = $this->populateParameterBag($filterData);

        $dataSet = $this->dataSourceManager->findBy($newFilterData);

        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if (isset($row['CDTIPOINSCRICAO']) && !empty($row['CDTIPOINSCRICAO'])) {
                if( $row['CDTIPOINSCRICAO'] == 'CPF' && isset($row['NRINSCRICAOPARC']) && !empty($row['NRINSCRICAOPARC']) && strlen($row['NRINSCRICAOPARC']) == 11 ) {
                    $cpf = preg_replace("/[^0-9]/", "", $row['NRINSCRICAOPARC']);
                    $cpf = str_pad($cpf, 11, "0", STR_PAD_LEFT);
                    $row['NRINSCRICAOPARC'] = Util::formatCnpjCpf($cpf);
                }
                elseif( $row['CDTIPOINSCRICAO'] == 'CNPJ' && isset($row['NRINSCRICAOPARC']) && !empty($row['NRINSCRICAOPARC']) && strlen($row['NRINSCRICAOPARC']) == 14 ){
                    $cnpj = preg_replace("/[^0-9]/", "", $row['NRINSCRICAOPARC']);
                    $cnpj = str_pad($cnpj, 14, "0", STR_PAD_LEFT);
                    $row['NRINSCRICAOPARC'] = Util::formatCnpjCpf($cnpj);
                }    
            }
            
            if (isset($row['NRESTRUTLEGAL']) && !empty($row['NRESTRUTLEGAL'])){
                $row['NRESTRUTLEGAL'] = explode(",",$row['NRESTRUTLEGAL']);
                $row['NMESTRUTLEGAL'] = explode(",",$row['NMESTRUTLEGAL']);
            }
            
            if (isset($row['NRESTRUTGEREN']) && !empty($row['NRESTRUTGEREN'])){
                $row['NRESTRUTGEREN'] = explode(",",$row['NRESTRUTGEREN']);
                $row['NMESTRUTGEREN'] = explode(",",$row['NMESTRUTGEREN']);
            }
            
            if (isset($row['NRESTRUTSIND']) && !empty($row['NRESTRUTSIND'])){
                $row['NRESTRUTSIND'] = explode(",",$row['NRESTRUTSIND']);
                $row['NMESTRUTSIND'] = explode(",",$row['NMESTRUTSIND']);
            }
            
            if (isset($row['NRTIPOVINCULOM']) && !empty($row['NRTIPOVINCULOM'])){
                $row['NRTIPOVINCULOM'] = explode(",",$row['NRTIPOVINCULOM']);
                $row['NMTIPOVINCULOH'] = explode(",",$row['NMTIPOVINCULOH']);
            }
        }    
        
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));    
    }
    
    public function salvarAssociacaoFinanceira (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (isset($row['NRESTRUTLEGAL'])){
                    if (is_array($row['NRESTRUTLEGAL'])){
                        if (empty($row['NRESTRUTLEGAL'])) {
                            $row['NRESTRUTLEGAL'] = null;
                        } else {
                            $row['NRESTRUTLEGAL'] = implode(",",$row['NRESTRUTLEGAL']);
                        }
                    }
                } else {
                    $row['NRESTRUTLEGAL'] = null;
                }
                
                if (isset($row['NRESTRUTGEREN'])){
                    if (is_array($row['NRESTRUTGEREN'])){
                        if (empty($row['NRESTRUTGEREN'])) {
                            $row['NRESTRUTGEREN'] = null;
                        } else {
                            $row['NRESTRUTGEREN'] = implode(",",$row['NRESTRUTGEREN']);
                        }
                    }
                } else {
                    $row['NRESTRUTGEREN'] = null;
                }
                
                if (isset($row['NRESTRUTSIND'])){
                    if (is_array($row['NRESTRUTSIND'])){
                        if (empty($row['NRESTRUTSIND'])) {
                            $row['NRESTRUTSIND'] = null;
                        } else {
                            $row['NRESTRUTSIND'] = implode(",",$row['NRESTRUTSIND']);
                        }
                    }
                } else {
                    $row['NRESTRUTSIND'] = null;
                }
                
                if (isset($row['NRTIPOVINCULOM'])){
                    if (is_array($row['NRTIPOVINCULOM'])){
                        if (empty($row['NRTIPOVINCULOM'])) {
                            $row['NRTIPOVINCULOM'] = null;
                        } else {
                            $row['NRTIPOVINCULOM'] = implode(",",$row['NRTIPOVINCULOM']);
                        }
                    }
                } else {
                    $row['NRTIPOVINCULOM'] = null;
                }
                
                if (array_key_exists('NRINCIDEPARCPAGAR', $row)) {
                    if ($row['NRINCIDEPARCPAGAR'] === '') {
                        $row['NRINCIDEPARCPAGAR'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'FPA_INCIDEPARCPAGAR', 1, 12, true);
                    }
                } else {
                    $row['NRINCIDEPARCPAGAR'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'FPA_INCIDEPARCPAGAR', 1, 12, true);
                }
            }
            $this->dataSourceName = $this->dataSourceName_Set;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAssociacaoFinanceira (DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 