<?php

namespace HCM\FPA\Controllers;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DataSource\DataSet;

class ReajusteController extends Crud {

    protected $dataSourceName = 'fpa_reajuste';
    protected $dataSourceName_get = 'fpa_reajuste_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_CDOPERINCLUSAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,\HCM\FPA\Services\ReajusteService $reajusteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->reajusteService = $reajusteService;
        parent::__construct($dataSourceManager);
    }

    public function getReajuste(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }  
            
            if($row['DSORDEM'] == 'TIPO_HIERARQUIA'){
                $row['NRTIPOHIERARQORG'] = explode(",", $row['DSINICIO']);
                foreach($row['NRTIPOHIERARQORG'] as $chave => $value)
                    $row['NRTIPOHIERARQORG'][$chave] = intval($value);
                
                $row['NMTIPOHIERARQORG'] = explode(",", $row['NMTIPOHIERARQORG']);
                
                if($row['DSINICIO2']){
                    $row['LIST_NRESTRUTURAMSUP'] = explode(",", $row['DSINICIO2']);
                    foreach($row['LIST_NRESTRUTURAMSUP'] as $chave => $value)
                        $row['LIST_NRESTRUTURAMSUP'][$chave] = intval($value);
                    
                    $row['NMESTRUTURAMSUP'] = explode(",", $row['NMESTRUTURAMSUP']);
                }
                if($row['DSFIM2']){
                    $row['LIST_NRESTRUTURAMREL'] = explode(",", $row['DSFIM2']);
                    foreach($row['LIST_NRESTRUTURAMREL'] as $chave => $value)
                        $row['LIST_NRESTRUTURAMREL'][$chave] = intval($value);
                    
                    $row['NMESTRUTURAMREL'] = explode(",", $row['NMESTRUTURAMREL']);
                }
            }  
            if($row['DSORDEM'] == 'LISTA_OCUPACAO'){
                $row['LIST_NROCUPACAOM'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NROCUPACAOM'] as $chave => $value)
                    $row['LIST_NROCUPACAOM'][$chave] = intval($value);
                    
                $row['NMOCUPACAOH'] = explode(",", $row['NMOCUPACAOH']);
            }

            if(!empty($row['LISTA_NRESTRUTSIND'])){
                $row['LISTA_NRESTRUTSIND'] = explode(",", $row['LISTA_NRESTRUTSIND']);
                
                $row['NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
            }
            
            if($row['NROCUPACAOM']){
                $row['NROCUPACAOM'] = explode(",", $row['NROCUPACAOM']);
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }
    
    public function setReajuste(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMALTSALAR', $row)) {
                    if ($row['NRFORMALTSALAR'] === '') {
                        $row['NRFORMALTSALAR'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_FORMALTSALAR', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMALTSALAR'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_FORMALTSALAR', 1, 12, true); 
                }
            }
            if(!array_key_exists('DSINICIO', $row)){
                $row['DSINICIO'] = null;
            }

            if(!array_key_exists('DSFIM', $row)){
                $row['DSFIM'] = null;
            }
            
            $row['DSFIM2'] = !array_key_exists('DSFIM2', $row) ? null : $row['DSFIM2'];
            $row['DSINICIO2'] = !array_key_exists('DSINICIO2', $row) ? null : $row['DSINICIO2'];
            
            
            if($row['DSORDEM'] === "LISTA_VINCULO"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }
            if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if($row['DSFIM'] && is_array($row['DSFIM'])){
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if($row['DSFIM'] && is_array($row['DSFIM'])){
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            if($row['DSORDEM'] == 'TIPO_HIERARQUIA'){
                if(array_key_exists('DSINICIO', $row) && is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if(array_key_exists('DSINICIO2', $row) && is_array($row['DSINICIO2'])){
                    $row['DSINICIO2'] = implode(',',$row['DSINICIO2']);
                }
                if(array_key_exists('DSFIM2', $row) && is_array($row['DSFIM2'])){
                    if(count($row['DSFIM2']) > 0){
                        $row['DSFIM2'] = implode(',',$row['DSFIM2']);
                    }else{
                        $row['DSFIM2'] = null;    
                    } 
                }
            }
            if($row['DSORDEM'] == 'LISTA_OCUPACAO'){
                if(array_key_exists('DSINICIO', $row) && is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }

            if(array_key_exists('LISTA_NRESTRUTSIND', $row) && is_array($row['LISTA_NRESTRUTSIND'])){
                if(count($row['LISTA_NRESTRUTSIND']) > 0){
                    $row['LISTA_NRESTRUTSIND'] = implode(',',$row['LISTA_NRESTRUTSIND']);
                }else{
                    $row['LISTA_NRESTRUTSIND'] = null;    
                } 
            }
            
            if(array_key_exists('NROCUPACAOM', $row) && is_array($row['NROCUPACAOM'])){
                if(count($row['NROCUPACAOM']) > 0){
                    $row['NROCUPACAOM'] = implode(',',$row['NROCUPACAOM']);
                }else{
                    $row['NROCUPACAOM'] = null;    
                } 
            }
            
            $row['NRTABELAFOLHA'] = isset($row['NRTABELAFOLHA'])?$row['NRTABELAFOLHA']:null; 
            $row['NRTIPOVINC'] = isset($row['NRTIPOVINC'])?$row['NRTIPOVINC']:null; 
            $row['NRVINCEMPREG'] = isset($row['NRVINCEMPREG'])?$row['NRVINCEMPREG']:null;
            $request->getDataSet()->setDataSourceName($this->dataSourceName);
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteReajuste(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function reajustarSalario(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $return = $this->reajusteService->reajustarSalario($row, $response);
            }
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getDemonstrativo', is_null($return)?array():$return));

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function retrocede(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $return = $this->reajusteService->retrocede($row);
            }
        } catch(\Exception $e) {
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    
} 