<?php
namespace HCM\Geral\Controllers;

use HCM\Geral\Services\PreFechamentoIrrfService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class PreFechamentoIrrfController {

    public $hcmParameters;
    public $preFechamentoIrrfService;
    public $retencaoTrabalhador;
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoIrrfService $preFechamentoIrrfService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoIrrfService = $preFechamentoIrrfService;
        $this->retencaoTrabalhador = new RetencaoTrabalhador($hcmParameters, $preFechamentoIrrfService);
    }
    
    public function getRetencaoTrabalhador(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->retencaoTrabalhador->getRetencaoTrabalhador($request->getDataSet()->getRows()[0], $response);
    }
    
    public function getEventosApuradosVinculoIrrf(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->retencaoTrabalhador->getEventosApuradosVinculoIrrf($request->getDataSet()->getRows()[0], $response);
    }
}

class RetencaoTrabalhador {
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoIrrfService $preFechamentoIrrfService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoIrrfService = $preFechamentoIrrfService;
    }
    
    public function getRetencaoTrabalhador($row, DTO\Response $response) {
        try {
            $parameters = array( 
                'NRORG'          => $row['NRORG'],
                'DTMESCOMPETENC' => $row['DATACOMPETENCIAINICIAL'],
                'NRTABELA'       => isset($row['APURACAO']) ? $row['APURACAO'] : null,
                'IDDIFERENCA'    => isset($row['IDDIFERENCA']) ? $row['IDDIFERENCA'] : '',
                'NRESTRUTLEGAL'  => isset($row['NRESTRUTLEGAL_LIST']) ? $row['NRESTRUTLEGAL_LIST'] : '',
                'NRESTRUTGEREN'  => isset($row['NRESTRUTGEREN_LIST']) ? $row['NRESTRUTGEREN_LIST'] : '',
                'NRESTRUTTOMAD'  => isset($row['NRESTRUTTOMAD_LIST']) ? $row['NRESTRUTTOMAD_LIST'] : '',
                'NRTIPOESTRUTGEREN' => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                'NRTIPOESTRUTTOMAD' => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            );

            $retencaoTrabalhador = $this->preFechamentoIrrfService->getRetencaoTrabalhador($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRetencaoTrabalhador', $retencaoTrabalhador));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getEventosApuradosVinculoIrrf($row, DTO\Response $response) {
        try {
            $parameters = array( 
                'NRORG'             => $this->hcmParameters->getNrorgTrabalho(),
                'COMPETENCIA'       => $row['DTOCORRENCIACALC'],
                'NRTIPOMOVIMENT'    => $row['NRTABELA'],
                'NRVINCULOM'        => $row['NRVINCULOM'],
                'DIFERENCA_BASE'    => $row['DIFERENCA_BASE'],
                'CONCLUSAO'         => $row['CONCLUSAO'],
                'MODALIDADECALCULO' => 1,
                'OCORRENCIACALCULO' => 1
            );
            
            $eventosApuradosVinculo = $this->preFechamentoIrrfService->getEventosApuradosVinculoIrrf($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('dataset', $eventosApuradosVinculo));
        } catch(\Exception $e) {
           $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode())); 
        }
    }
}