<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class PreFechamentoIrrfRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->retencaoTrabalhador = new RetencaoTrabalhador($entityManager);
    }
        
    public function getRetencaoTrabalhador($parameters) {
        return $this->retencaoTrabalhador->getRetencaoTrabalhador($parameters);
    }
    
    public function getEventosApuradosVinculoIrrf($parameters) {
        return $this->retencaoTrabalhador->getEventosApuradosVinculoIrrf($parameters);
    }
}

class RetencaoTrabalhador {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getRetencaoTrabalhador($parameters) {
      $sql = $this->entityManager->getConnection()->prepare(Queries::$queryRetencaoTrabalhador);
        $sql->execute(
            array(
                'P_NRORG'          => $parameters['NRORG'],
                'P_DTMESCOMPETENC' => $parameters['DTMESCOMPETENC'],
                'P_NRTABELA'       => $parameters['NRTABELA'],
                'P_IDDIFERENCA'    => $parameters['IDDIFERENCA'],
                'P_MODALIDADECAL'  => 1,
                'P_NRESTRUTLEGAL'  => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRESTRUTGEREN'  => is_array($parameters['NRESTRUTGEREN']) ? implode(",",$parameters['NRESTRUTGEREN']) : $parameters['NRESTRUTGEREN'],
                'P_NRESTRUTTOMAD'  => is_array($parameters['NRESTRUTTOMAD']) ? implode(",",$parameters['NRESTRUTTOMAD']) : $parameters['NRESTRUTTOMAD'],
                'P_NRVINCULO'      => null,
                'P_NRTIPOESTRUTGEREN' => $parameters['NRTIPOESTRUTGEREN'],
                'P_NRTIPOESTRUTTOMAD' => $parameters['NRTIPOESTRUTTOMAD'],
            )
        );
        return $sql->fetchAll();
    }
    
    public function getEventosApuradosVinculoIrrf($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$queryEventosApuradosVinculoIrrf);
        $sql->execute(
            array( 
                'P_NRORG'             => $parameters['NRORG'],
                'P_COMPETENCIA'       => $parameters['COMPETENCIA'],
                'P_NRVINCULOM'        => $parameters['NRVINCULOM'],
                'P_MODALIDADECALCULO' => $parameters['MODALIDADECALCULO'],
                'P_NRTIPOMOVIMENT'    => $parameters['NRTIPOMOVIMENT']
            )
        );
        return $sql->fetchAll();
    }
}

class Queries {
    
    static $queryRetencaoTrabalhador = "
SELECT * FROM (
  SELECT
      RES.NRORG,
      RES.NRPESSOA,
      RES.NRCPFPESSOA,
      RES.NRCPFPESSOA||' - '||RES.NMVINCULOM PESSOA,
      RES.NRVINCULOM,
      RES.NMVINCULOM,
      RES.DTADMISSAOVINC,
      RES.DTRESCISAOVINC,
      RES.CDMATRICULAESOCIAL,
      NVL(RES.CDMATRICULAESOCIAL, RES.NRVINCULOM) MATRICULA,
      RES.NRTIPOVINCULOM,
      RES.CDESOCIAL CDCATEGESOC,
      RES.DSCATEGESOC,
      RES.NRESTRUTLEGAL,
      RES.NMESTRUTLEGAL,
      RES.CDCNPJESTRUT,
      SUBSTR(RES.CDCNPJESTRUT, 0,8) CNPJRAIZ,
      RES.NRESTRUTGEREN,
      RES.NMESTRUTGEREN,
      RES.NRESTRUTTOMAD,
      RES.NMESTRUTTOMAD,
      RES.CDCNPJTOMADOR,
      TO_CHAR(RES.DTOCORRENCIACALC, 'DD/MM/YYYY') DTOCORRENCIACALC,
      TO_CHAR(RES.DTPAGAMENTO, 'DD/MM/YYYY') DTPAGAMENTO,
      RES.NRCALCULOFOLHA,
      RES.NMTIPOMOVIMENT,
      RES.NRTPAMBIENTE,
      RES.VRBASE,
      RES.VRIRRF,
      ESOPE.IDSTATUS,
      ESOPE.NRTABELA,
      ESOPE.NMEVENTO,
      RETIR.NRENVIO,
      RETIR.IDEDMDEV,
      SUM(NVL(RETIR.VRBASEMENSAL,0) + NVL(RETIR.VRBASE13,0)) VRBASE_ESO,
      SUM(NVL(RETIR.VRVALORMENSAL,0) + NVL(RETIR.VRVALOR13,0)) VRIRRF_ESO,
      RES.VRBASE - SUM(NVL(RETIR.VRBASEMENSAL,0) + NVL(RETIR.VRBASE13,0)) DIFERENCA_BASE,
      RES.VRIRRF - SUM(NVL(RETIR.VRVALORMENSAL,0) + NVL(RETIR.VRVALOR13,0)) DIFERENCA_IRRF
    FROM (SELECT
              VM.NRORG,
              VM.NRPESSOA,
              PE.NRCPFPESSOA,
              VM.NRVINCULOM,
              VM.NMVINCULOM,
              VM.DTADMISSAOVINC,
              VM.DTRESCISAOVINC,
              VH.CDMATRICULAESOCIAL,
              VM.NRTIPOVINCULOM,
              CATEGTRAB.CDESOCIAL,
              CATEGTRAB.DSCATEGESOC,
              VH.NRESTRUTLEGAL,
              EHL.NMRAZSOCESTRUT NMESTRUTLEGAL,
              EHL.CDCNPJESTRUT,
              VH.NRESTRUTGEREN,
              EHG.NMESTRUTURAH NMESTRUTGEREN,
              ETM.NRESTRUTURAM NRESTRUTTOMAD,
              ETH.NMRAZSOCESTRUT NMESTRUTTOMAD,
              ETH.CDCNPJESTRUT CDCNPJTOMADOR,
              CF.DTOCORRENCIA DTOCORRENCIACALC,
              PAG.DTOCORRENCIA DTPAGAMENTO,
              PAG.NRCALCULOFOLHA,
              TM.NMTIPOMOVIMENT,
              /*ICF.NREVENTOM,*/
              SUM(CASE WHEN ICF.NREVENTOM IN (3080,3110,3140,3170,3175,3200,3212) THEN ICF.VREVENTOFOLHA ELSE 0 END) VRBASE,
              SUM(CASE WHEN ICF.NREVENTOM IN (1330,1340,1350,1360,1370,1380,1390,1395) THEN ICF.VREVENTOFOLHA ELSE 0 END) VRIRRF,
              EP.NRTPAMBIENTE
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                AND VH.NRVINCULOM = VM.NRVINCULOM
                                AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                          WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                            AND NRORG      = VM.NRORG
                                                            AND NRVINCULOM = VM.NRVINCULOM )
  
            JOIN GPE_PESSOAH PE ON PE.NRPESSOA = VM.NRPESSOA 
                               AND PE.NRORG = VM.NRORG 
                               AND PE.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH 
                                                         WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND NRORG = PE.NRORG
                                                           AND NRPESSOA = PE.NRPESSOA)
            /*CATEGORIA*/
            JOIN GPE_VINCULOEMPREG VINCEMPREG ON VINCEMPREG.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
            JOIN GPE_CATEGTRABESOC CATEGTRAB ON CATEGTRAB.NRCATEGESOC = VINCEMPREG.NRCATEGESOC
            /*ESTRUTURA LEGAL*/
            JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                    AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
            LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                    AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                    AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                              WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                AND NRORG        = VH.NRORG
                                                                AND NRESTRUTURAM = EML.NRESTRUTURAM )
            /*ESTRUTURA GERENCIAL*/
            JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                    AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
            LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                    AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                    AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                              WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                AND NRORG        = VH.NRORG
                                                                AND NRESTRUTURAM = EMG.NRESTRUTURAM )
            /*ESTRUTURA TOMADOR*/                                    
            LEFT JOIN TIPOHIERARQORG TH ON (TH.NRORG = VH.NRORG OR TH.NRORG = 0)
                  AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                  AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTTOMAD, 7)
            LEFT JOIN HIERARQUIAORG HO ON HO.NRORG = VH.NRORG
                                      AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
                                      AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
                                      AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                                      AND (HO.DTFIMVIGENCIA IS NULL OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
                                      AND HO.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA)
                                                                FROM HIERARQUIAORG HI
                                                              WHERE HI.NRORG = HO.NRORG
                                                                AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
                                                                AND HI.NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
                                                                AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                                                                AND (HI.DTFIMVIGENCIA IS NULL OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
            LEFT JOIN ESTRUTURAM ETM ON ETM.NRORG = HO.NRORG
                                    AND ETM.NRESTRUTURAM = HO.NRESTRUTURAMSUP
            LEFT JOIN ESTRUTURAH ETH ON ETH.NRORG = ETM.NRORG
                                    AND ETH.NRESTRUTURAM = ETM.NRESTRUTURAM
                                    AND ETH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                              WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                AND NRORG        = ETH.NRORG
                                                                AND NRESTRUTURAM = ETH.NRESTRUTURAM )
            /*CALCULO*/
            JOIN FPA_ITECALCFOLHA PAG ON PAG.NRORG = VM.NRORG
                                     AND PAG.NRVINCULOM  = VM.NRVINCULOM
                                     AND PAG.NREVENTOM IN (3030,3050)
                                     AND TRUNC(PAG.DTOCORRENCIA,'MM') = :P_DTMESCOMPETENC
            JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = PAG.NRORG
                                     AND ICF.NRCALCULOFOLHA = PAG.NRCALCULOFOLHA
                                     AND ICF.NRVINCULOM  = VM.NRVINCULOM
                                     AND ICF.NREVENTOM IN (1330,1340,1350,1360,1370,1380,1390,1395,3080,3110,3140,3170,3175,3200,3212)
            JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                    AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                    AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                    AND ((CF.NRTIPOMOVIMENT IN (1,2,10,19) AND :P_NRTABELA = 1) OR
                                         (CF.NRTIPOMOVIMENT = 4 AND :P_NRTABELA = 2) OR
                                         (CF.NRTIPOMOVIMENT IN (1,2,4,10,19) AND :P_NRTABELA IS NULL))
            JOIN FPA_TIPOMOVIMENT TM ON TM.NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT
            /*PARAMETRO ESOCIAL*/
            JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG
                           AND EP.NRESTRUTURAM = EML.NRESTRUTURAM
            /**/
            WHERE VM.NRORG = :P_NRORG
              AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC) )
              AND ( VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC OR VM.DTRESCISAOVINC IS NULL )
              AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
              AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
              AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL )
              AND ( INSTR(','||:P_NRESTRUTTOMAD||',', ','||ETM.NRESTRUTURAM||',' ) > 0 OR :P_NRESTRUTTOMAD IS NULL )
            GROUP BY
              VM.NRORG,
              VM.NRPESSOA,
              PE.NRCPFPESSOA,
              VM.NRVINCULOM,
              VM.NMVINCULOM,
              VM.DTADMISSAOVINC,
              VM.DTRESCISAOVINC,
              VH.CDMATRICULAESOCIAL,
              VM.NRTIPOVINCULOM,
              CATEGTRAB.CDESOCIAL,
              CATEGTRAB.DSCATEGESOC,
              VH.NRESTRUTLEGAL,
              EHL.NMRAZSOCESTRUT,
              EHL.CDCNPJESTRUT,
              VH.NRESTRUTGEREN,
              EHG.NMESTRUTURAH,
              ETM.NRESTRUTURAM,
              ETH.NMRAZSOCESTRUT,
              ETH.CDCNPJESTRUT,
              CF.DTOCORRENCIA,
              PAG.DTOCORRENCIA,
              PAG.NRCALCULOFOLHA,
              TM.NMTIPOMOVIMENT,
              /*ICF.NREVENTOM,*/
              EP.NRTPAMBIENTE
          ) RES
  
    LEFT JOIN ESO_PERIODICO ESOPE ON ESOPE.NRORG = RES.NRORG
                                 AND ESOPE.NRTPAMBIENTE = RES.NRTPAMBIENTE
                                 AND ESOPE.NMEVENTO = 'S-1210'
                                 AND ESOPE.DTMESCOMPETENC = :P_DTMESCOMPETENC
                                 AND ESOPE.NRCHAVE = RES.NRPESSOA
                                 AND ESOPE.IDATIVO = 'S'
                                 AND ESOPE.NRENVIOEXCL IS NULL
                                 AND ESOPE.IDSTATUS IN ('C','A')
  
    LEFT JOIN ESO_PERIODRETIRRF RETIR ON RETIR.NRORG = ESOPE.NRORG
                                     AND RETIR.NRENVIO = ESOPE.NRENVIO
                                     AND RETIR.CDCATEGTRAB = RES.CDESOCIAL
                                     AND RETIR.IDEDMDEV = RES.NRTIPOVINCULOM||'.'||RES.CDESOCIAL||'.'||RES.NRCALCULOFOLHA
  
    LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG = ESOPE.NRORG
                            AND EMP.NRESTRUTURAM = ESOPE.NRESTRUTURAM
                            AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CDCNPJESTRUT,1,8)
                            AND EMP.CDCNPJESTRUT IS NOT NULL
                            AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                        WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND NRORG = EMP.NRORG
                                                          AND NRESTRUTURAM = EMP.NRESTRUTURAM )
  
   GROUP BY
      RES.NRORG,
      RES.NRPESSOA,
      RES.NRVINCULOM,
      RES.NMVINCULOM,
      RES.DTADMISSAOVINC,
      RES.DTRESCISAOVINC,
      RES.CDMATRICULAESOCIAL,
      RES.NRTIPOVINCULOM,
      RES.CDESOCIAL,
      RES.DSCATEGESOC,
      RES.NRCPFPESSOA,
      RES.NRESTRUTLEGAL,
      RES.NMESTRUTLEGAL,
      RES.CDCNPJESTRUT,
      SUBSTR(RES.CDCNPJESTRUT, 0,8),
      RES.NRESTRUTGEREN,
      RES.NMESTRUTGEREN,
      RES.NRESTRUTTOMAD,
      RES.NMESTRUTTOMAD,
      RES.CDCNPJTOMADOR,
      TO_CHAR(RES.DTOCORRENCIACALC, 'DD/MM/YYYY'),
      TO_CHAR(RES.DTPAGAMENTO, 'DD/MM/YYYY'),
      RES.NRCALCULOFOLHA,
      RES.NMTIPOMOVIMENT,
      RES.NRTPAMBIENTE,
      RES.VRBASE,
      RES.VRIRRF,
      ESOPE.IDSTATUS,
      ESOPE.NRTABELA,
      ESOPE.NMEVENTO,
      RETIR.NRENVIO,
      RETIR.IDEDMDEV
    )
WHERE ( ( :P_IDDIFERENCA = 'S' AND ( ABS(DIFERENCA_BASE) > 0.05 OR ABS(DIFERENCA_IRRF) > 0.05 ) ) OR :P_IDDIFERENCA = 'N' )
";
    
    static $queryEventosApuradosVinculoIrrf = "
SELECT
      ICF.VREVENTOFOLHA VALOR,
      SUBSTR(ICF.DTOCORRENCIA, 4, 7) PERIODO,
      DECODE(
        CF.NRTIPOMOVIMENT, 1, 'Mensal',
                          4, '13º'
      ) PROCESSO,
      EH.NREVENTOM NREVENTOM,
      EH.NMEVENTOH EVENTO,              
      DECODE(EH.NRNATURUBR, NULL, NULL,
      NVL((SELECT MAX(IRF.CDESOCIAL)
              FROM INCIDENCIAIRRF IRF
              JOIN FPA_PROCINCIDEVENTO EVT ON IRF.NRINCIDIRRF = EVT.NRINCIDIRRF
            WHERE EVT.NRORG = ICF.NRORG
              AND EVT.NREVENTOM = ICF.NREVENTOM 
              AND EVT.NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT), DECODE(EM.NREVENTOM, 3080, '79', NVL(IRRF.CDESOCIAL,'9')))) CDINCIDEIRRF,
      DECODE(EH.NRNATURUBR, NULL, 'Sem parametrização',
        DECODE(EH.NREVENTOM, 3080, 'Não Incide',
          DECODE(
            IRRF.CDESOCIAL, '0', 'Não Incide', '1', 'Não Incide', '9', 'Não Incide', '79', 'Não Incide',
                            '11', 'Incide Base', '12', 'Incide Base', '13', 'Incide Base', '14', 'Incide Base', '15', 'Incide Base',
                            '31', 'Incide Imposto', '32', 'Incide Imposto', '33', 'Incide Imposto', '34', 'Incide Imposto', '35', 'Incide Imposto',
                            NULL, 'Não Incide',
                            'Base, Imposto ou Encargo'
          )
        )
      ) INCIDEIRRF
FROM FPA_ITECALCFOLHA ICF
JOIN FPA_CALCULOFOLHA CF 
  ON CF.NRORG = ICF.NRORG
  AND CF.NRCALCULOFOLHA  = ICF.NRCALCULOFOLHA
  AND CF.NRTPMODALIDCAL  = :P_MODALIDADECALCULO
  AND CF.DTOCORRENCIA    = :P_COMPETENCIA
  AND CF.NRTIPOMOVIMENT  = :P_NRTIPOMOVIMENT
JOIN FPA_EVENTOM EM ON (EM.NREVENTOM = ICF.NREVENTOM)
JOIN FPA_EVENTOH EH ON (EH.NREVENTOM = EM.NREVENTOM)
JOIN (SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC, NREVENTOM
        FROM FPA_EVENTOH 
        WHERE DTMESCOMPETENC <= :P_COMPETENCIA
        GROUP BY NREVENTOM) EHATUAL 
          ON ( EHATUAL.NREVENTOM = EH.NREVENTOM
          AND EHATUAL.DTMESCOMPETENC = EH.DTMESCOMPETENC)
/*IRRF NOVO*/
LEFT JOIN (SELECT IE.NREVENTOM, REPLACE(TI.DSNRTIPOMOVIMENT, ' ', '') AS DSNRTIPOMOVIMENT, MAX(INC.CDESOCIAL) AS CDESOCIAL
              FROM FPA_INCIDEVENTO IE
              JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
              JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
                                    AND (INC.NRORG = :P_NRORG OR INC.NRORG = 0)
            WHERE (IE.NRORG = :P_NRORG OR IE.NRORG = 0)
                    AND (IE.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                              FROM FPA_GRPITEM 
                                              WHERE NRGRPINCIDE = 47) OR TI.DSNRTIPOMOVIMENT IS NOT NULL)
          GROUP BY IE.NREVENTOM, TI.DSNRTIPOMOVIMENT) IRRF ON IRRF.NREVENTOM = ICF.NREVENTOM
                                                          AND INSTR(','||IRRF.DSNRTIPOMOVIMENT||',',','||CF.NRTIPOMOVIMENT||',') > 0
WHERE ICF.NRORG      = :P_NRORG
  AND ICF.NRVINCULOM = :P_NRVINCULOM
    ";

}