<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 02/09/2016
 * Time: 11:14
 */

namespace HCM\VAB\Services;

use HCM\VAB\Repository\LancamentoBeneficioRepository;
use Zeedhi\Framework\HTTP\Kernel;
use Doctrine\ORM\EntityManager;
use Folha\Entities\FpaItecalcfolha;
use Folha\Entities\FpaTipoassoceve;
use Folha\Entities\FpaTipomoviment;
use Folha\Entities\VabHistbenefvinc;
use Folha\Entities\VabBeneficiario;
use Folha\Repositories\FpaEventoh;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Geral\Util\NovoCodigo;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\VAB\Repository\HistbenefvincRepository;
use HCM\FPA\Services\IteCalcFolhaService;
use HCM\Util\FPA;
use HCM\Util\Valor;

class LancamentoBeneficioService {

    protected $hcmParameters;

    private $dslotelancevento       = 'EXPORTAÇÃO DO LANÇAMENTO DE BENEFÍCIOS';
    private $dsLoteLanceventoMensal = 'EXPORTAÇÃO DE ITENS DO MOVIMENTO DE BENEFÍCIO';
    private $idstatuslotelanc       = 'PENDENTE';
    private $idoriglotelanc         = 'VAB';

    public function __construct(Kernel $httpKernel, EntityManager $entityManager, LancamentoBeneficioRepository $lancamentoBeneficioRepository, HCMParameters $hcmParameters, LoteLancEventoService $loteLancEventoService, 
                                HistbenefvincRepository $histbenefvincRepository, IteCalcFolhaService $iteCalcFolhaService) {
        $this->httpKernel              = $httpKernel;          
        $this->entityManager           = $entityManager;
        $this->lancamentoBeneficioRepository = $lancamentoBeneficioRepository;
        $this->hcmParameters           = $hcmParameters;
        $this->loteLancEventoService   = $loteLancEventoService;
        $this->histbenefvincRepository = $histbenefvincRepository;
        $this->iteCalcFolhaService     = $iteCalcFolhaService;
    }

    public function verificaEventoVinculoExportadoFolha($nrcalculofolha, $nrvinculom, $nreventom, $nrorg) {
        try {
            $params = array(
                'P_NRCALCULOFOLHA' => $nrcalculofolha,
                'P_NRVINCULOM'     => $nrvinculom,
                'P_NREVENTOM'      => $nreventom,
                'P_NRORG'          => $nrorg);
            $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM AND NREVENTOM = :P_NREVENTOM";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
            if($itemcalcs){
                throw new \Exception(Messages::HCM02000_VAB_EVEN_EXPORTADO_FOLHA_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaVinculoLiquidoGravado($nrcalculofolha, $nrvinculom, $nrorg) {
        try {
            $params = array(
                'P_NRCALCULOFOLHA' => $nrcalculofolha,
                'P_NRVINCULOM'     => $nrvinculom,
                'P_NRORG'          => $nrorg);
            $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
            if($itemcalcs){
                /** @var FpaItecalcfolha $itemcalc */
                foreach($itemcalcs as $itemcalc){
                    if($itemcalc->getNreventom() == FpaTipoassoceve::ASSOC_LIQUIDO){
                        throw new \Exception(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Messages::ERROR_MESSAGE_CODE);
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function existeEvento($nrcalculofolha, $nrvinculom, $nrorg, $evento ) {
        try {
            $params = array(
                'P_NRCALCULOFOLHA' => $nrcalculofolha,
                'P_NRVINCULOM'     => $nrvinculom,
                'P_NRORG'          => $nrorg);
            $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
            if($itemcalcs){
                /** @var FpaItecalcfolha $itemcalc */
                foreach($itemcalcs as $itemcalc){
                    if($itemcalc->getNreventom() == $evento){
                        return $itemcalc;
                    }
                }
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function existeEventoCopEmpresa($nrcalculofolha, $nrvinculom, $nrorg, $evento, $dtmescompetenc ) {
        try {
            $params = array(
                'P_NRCALCULOFOLHA' => $nrcalculofolha,
                'P_NRVINCULOM'     => $nrvinculom,
                'P_NRORG'          => $nrorg);
            $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
            if($itemcalcs){
                foreach($itemcalcs as $itemcalc){
                    if($itemcalc->getNreventom() == $evento){
                        if($itemcalc->getDtocorrencia() == DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true))){
                            return $itemcalc;
                        }
                    }
                }
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom) {
        try {
            $itemcalcs = $this->histbenefvincRepository->getItensFolhaPorEventoHistorico($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
            if(!empty($itemcalcs)){
                throw new \Exception(Messages::HCM02000_VAB_VINC_EXPORTADO_FOLHA_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaLiquidoGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            $itemcalcs = $this->histbenefvincRepository->getItensHistoricoBeneficio(Fpatipoassoceve::ASSOC_LIQUIDO, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
            if ($itemcalcs) {
                throw new \Exception(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaParcialGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            $itemcalcs = $this->histbenefvincRepository->getItensHistoricoBeneficio(Fpatipoassoceve::ASSOC_LIQUIDO_PARCIAL, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
            if ($itemcalcs) {
                throw new \Exception(Messages::HCM01000_FPA_VINC_PARCIAL_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravarTotal($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                                $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            // $this->verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
            $this->insereIteCalcFolha(Fpatipoassoceve::ASSOC_LIQUIDO, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravarParcial($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                                  $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            // $this->verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
            $this->verificaLiquidoGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
            $this->insereIteCalcFolha(Fpatipoassoceve::ASSOC_LIQUIDO_PARCIAL, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cadastroDias($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                                  $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $qtdeDiasAdicionar, $qtdeDiasDiminuir) {
        try {
            // $this->verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
            $this->verificaLiquidoGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, null, null, null, null);
            $historicos = $this->histbenefvincRepository->getVinculosBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, null, null, null);
            
            foreach($historicos as $historico){
                $hist = $this->entityManager->getRepository(Repositories::VAB_HISTBENEFVINC)->findOneBy(['nrhistbenefvinc'=>intval($historico['NRHISTBENEFVINC']), 'nrorg'=>$this->hcmParameters->getNrorgTrabalho()]);
                
                $qtdeDiasTotais = $hist->getQtdiasbenef() + $qtdeDiasAdicionar - $qtdeDiasDiminuir;
    
                // Arredondar o valor do evento do beneficiário depois de multiplicar pela quantidade de dias.
                $objEventBenef = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                    $historico['NREVENTOM'], 
                    $this->hcmParameters->getNrorgTrabalho(), 
                    $this->hcmParameters->getNrorgPadrao(), 
                    $this->hcmParameters->getDtCompetencia() 
                );
                if($objEventBenef){
                    $vlrEventBeneficiario = Valor::arredonda(
                        $qtdeDiasTotais * $hist->getQtapurbenef() * $hist->getVrbeneficio(), 
                        $objEventBenef->historico->getIdtipoarredonda(), 
                        $objEventBenef->historico->getNrdigarredonda(),
                        $objEventBenef->historico->getVrlimitearred()
                    );
                }
                
                $hist->setQtdiasbenef($qtdeDiasTotais);
                $hist->setVrtotalbenef($vlrEventBeneficiario);
                $this->entityManager->persist($hist);
				$this->entityManager->flush($hist);
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retrocederItens($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                                     $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            // $this->verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
            $this->excluiItensBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportarLote($nrorg, $dtmescompetenc, $nrcalculobenef,
                                 $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            $this->verificaParcialGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
            $itens = $this->histbenefvincRepository->getHistoricosItens($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
            
            if (empty($itens)) {
                throw new \Exception(Messages::HCM02000_VAB_ITENS_BENEF_VAZIO_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            //Gera lote de eventos
            $this->loteLancEventoService->geraLoteLancEvento(
                $nrorg, $dtmescompetenc, $this->idoriglotelanc, $this->dslotelancevento, $this->idstatuslotelanc, '', FpaTipomoviment::TP_MOV_MENSAL, $itens
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportarLoteItensBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef,
                                                $nrestrutlegal, $nrestrutgeren, $nrestrutura_beneficio, $lista_nrvinculom, $row) {
        try {
            
            $itens = $this->iteCalcFolhaService->getItensCalc($nrcalculobenef, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrestrutura_beneficio, $lista_nrvinculom);
            if (empty($itens)) {
                throw new \Exception(Messages::HCM02000_VAB_ITENS_BENEF_VAZIO_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            
            $this->validaNomeLoteAExportar($lista_nrvinculom, $nrestrutura_beneficio, $nrestrutgeren, $nrestrutlegal, $row);
            
            //Gera lote de eventos
            $this->loteLancEventoService->geraLoteLancEvento(
                $nrorg, $dtmescompetenc, $this->idoriglotelanc, $this->dsLoteLanceventoMensal, $this->idstatuslotelanc, '', FpaTipomoviment::TP_MOV_MENSAL, $itens
            );
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportarLoteItensBeneficioListas($nrorg, $dtmescompetenc, $nrcalculobenef, $lista_nrestrutlegal, $lista_nrestrutgeren, $lista_nrestrutsind, $lista_nrvinculom,
                                                     $lista_nrestrutbenef, $lista_nrbeneficiom, $lista_nrtpbeneficio, $lista_nreventom, $row) {
        try {
            
            $itens = $this->iteCalcFolhaService->getItensCalcListas($nrcalculobenef, $nrorg, $dtmescompetenc, $lista_nrestrutlegal, $lista_nrestrutgeren, $lista_nrestrutsind, $lista_nrvinculom,
                                                                    $lista_nrestrutbenef, $lista_nrbeneficiom, $lista_nrtpbeneficio, $lista_nreventom);
            if (empty($itens)) {
                throw new \Exception(Messages::HCM02000_VAB_ITENS_BENEF_VAZIO_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            
            //$this->validaNomeLoteAExportar($lista_nrvinculom, $nrestrutura_beneficio, $nrestrutgeren, $nrestrutlegal, $row);
            
            //Gera lote de eventos
            $dsobslote = "Estrutura Legal: $lista_nrestrutlegal,
Estrutura Gerencial: $lista_nrestrutgeren,
Estrutura Sindical: $lista_nrestrutsind,
Vínculos: $lista_nrvinculom,
Estrutura do Benefício: $lista_nrestrutbenef,
Tipo de Benefício: $lista_nrtpbeneficio,
Benefícios: $lista_nrbeneficiom,
Eventos: $lista_nreventom";
            $this->loteLancEventoService->geraLoteLancEvento(
                $nrorg, $dtmescompetenc, $this->idoriglotelanc, $this->dsLoteLanceventoMensal, $this->idstatuslotelanc, $dsobslote, FpaTipomoviment::TP_MOV_MENSAL, $itens
            );
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaNomeLoteAExportar($lista_nrvinculom, $nrestrutura_beneficio, $nrestrutgeren, $nrestrutlegal, $row){
        if( $lista_nrvinculom !== "A" )
            $this->dsLoteLanceventoMensal .= ": LISTA DE VÍNCULOS";
        
        else if( isset($nrestrutura_beneficio) && !empty($nrestrutura_beneficio) )
            $this->dsLoteLanceventoMensal .= ": ".strtoupper($row['NMESTRUTURA_BENEFICIO']);
        
        else if( isset($nrestrutgeren) &&!empty($nrestrutgeren) )
            $this->dsLoteLanceventoMensal .= ": ".strtoupper($row['NMESTRUTGEREN']);
        
        else if( isset($nrestrutlegal) && !empty($nrestrutlegal) )
            $this->dsLoteLanceventoMensal .= ": ".strtoupper($row['NMESTRUTLEGAL']);
    }

    public function insereIteCalcFolha($nrevento, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal)
    {   
        //Deleta todos os itens cálculo vínculo de eventos 3040
        $this->excluiItensBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        $this->entityManager->flush();

        //Busca todos o líquidos gravados para o movimento de benefício
        $params = array(
            'P_NRORG'           => $nrorg,
            'P_NRCALCULOFOLHA'  => $nrcalculobenef,
            'P_NREVENTOM'       => FpaTipoassoceve::ASSOC_LIQUIDO
        );
        $where = "NREVENTOM = :P_NREVENTOM AND
                  NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND
                  NRORG = :P_NRORG";
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        $vinculosLiquido = array(0);
        /** @var FpaItecalcfolha $itemcalc */
        foreach($itemcalcs as $itemcalc){
            $vinculosLiquido[] = $itemcalc->getNrvinculom();
        }
        $vinculosLiquido = implode(', ', $vinculosLiquido);

        $liquidos = $this->histbenefvincRepository->getVinculosLiquidoBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $vinculosLiquido, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        $eventos  = $this->histbenefvincRepository->getVinculosBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
        
        foreach($liquidos as $liquido){
            $this->insereBancoIteCalcFolha($nrorg, $dtmescompetenc, $nrcalculobenef, $nrevento, $liquido, true);
        }
        foreach($eventos as $evento){
            $this->insereBancoIteCalcFolha($nrorg, $dtmescompetenc, $nrcalculobenef, $evento['NREVENTOM'], $evento);
        }
        $this->entityManager->flush();
    }

    private function insereBancoIteCalcFolha($nrorg, $dtmescompetenc, $nrcalculobenef, $nrevento, $info, $liquido = false)
    {
        try {
            if (($liquido) or (isset($info['DTPGTOBENEF']))) {
                $dtocorrencia = $info['DTPGTOBENEF'] == '' ? $dtmescompetenc : $info['DTPGTOBENEF'];
                $params = array(
                    'P_NRORG' => $nrorg,
                    'P_NRVINCULOM'     => $info['NRVINCULOM'],
                    'P_NRCALCULOFOLHA' => $nrcalculobenef,
                    'P_NREVENTOM'      => $nrevento,
                    'P_DTOCORRENCIA'   => DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO, true));
            } else {
                $params = array(
                    'P_NRORG' => $nrorg,
                    'P_NRVINCULOM'     => $info['NRVINCULOM'],
                    'P_NRCALCULOFOLHA' => $nrcalculobenef,
                    'P_NREVENTOM'      => $nrevento,
                    'P_DTOCORRENCIA'   => DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true));
            }

            $where = "NRORG        = :P_NRORG
                AND NRVINCULOM     = :P_NRVINCULOM
                AND NREVENTOM      = :P_NREVENTOM
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                AND DTOCORRENCIA   = :P_DTOCORRENCIA";
            $uniqueitem = FpaItecalcfolha::getObjectByWhere($where, $params);
            if (!$uniqueitem) {
                $ideventodemonst = FpaEventoh::getIdeventodemonst($nrevento, $dtmescompetenc);

                $itemcalc = new FpaItecalcfolha();
                $itemcalc->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $itemcalc->setCdoperultatu($this->hcmParameters->getCdoperador());
                $itemcalc->setDtinclusao(DateUtil::getDataAtual());
                $itemcalc->setDtultatu(DateUtil::getDataAtual());
                $itemcalc->setIdevendemoncalc($ideventodemonst);
                $itemcalc->setIdrefevalorcalc('NORMAL');
                $itemcalc->setNrcalculofolha($nrcalculobenef);
                $itemcalc->setNreventom($nrevento);
                $itemcalc->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $itemcalc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $itemcalc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $itemcalc->setNrvinculom($info['NRVINCULOM']);
                $itemcalc->setQtocorproxperi(1);
                $itemcalc->setVreventofolha($info['VREVENTO']);
                $itemcalc->setVrreffolha($info['QTDIASBENEF']);
                if (($liquido) or (isset($info['DTPGTOBENEF']))) {
                    $dtocorrencia = $info['DTPGTOBENEF'] == '' ? $dtmescompetenc : $info['DTPGTOBENEF'];
                    $itemcalc->setDtocorrencia(DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO, true));
                } else {
                    $itemcalc->setDtocorrencia(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true));
                }
                $itemcalc->setNritemcalcfolha(NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 21, true));
                $itemcalc->persist();
            } else {
                $uniqueitem->setVreventofolha($info['VREVENTO']);
                $uniqueitem->setVrreffolha($info['QTDIASBENEF']);
                $uniqueitem->persist();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function excluiItensBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal) {
        try {
            $params = array(
                'P_NRORG'            => $nrorg,
                'P_NRCALCULOFOLHA'   => $nrcalculobenef,
                'P_DTMESCOMPETENC'   => $dtmescompetenc,
                'P_NRBENEFICIOM'     => $nrbeneficiom,
                'P_NRTPBENEFICIO'    => $nrtpbeneficio,
                'P_NRESTRUTLEGAL'    => $nrestrutlegal,
                'P_NRESTRUTGEREN'    => $nrestrutgeren,
                'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
                'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
                'P_DTINIADMISSAO' => $dtiniadmissao,
                'P_DTFIMADMISSAO' => $dtfimadmissao,
                'P_NRESTRUTLEGAL_LIST' => $lista_nrestrutlegal
            );

            $condTpBeneficio = null;
            $condEstrutura = null;
            $condAdmissao = null;

            if($nrtpbeneficio){
                $condTpBeneficio = " AND NRVINCULOM IN (SELECT BE.NRVINCULOM
                                 FROM VAB_BENEFICIARIO BE
                                 JOIN VAB_BENEFICIOH BH ON BH.NRORG = :P_NRORG
                                                       AND (BH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
                                                       AND BH.TPPERIODOBENEF = 'DIARIO'
                                                       AND BH.NRBENEFICIOM = BE.NRBENEFICIOM
                                                       AND BH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH
                                                                                 WHERE NRORG = BH.NRORG
                                                                                   AND NRBENEFICIOM = BH.NRBENEFICIOM
                                                                                   AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                                WHERE BE.NRORG = :P_NRORG
                                  AND (BE.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM IS NULL)
                                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||BE.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A'))";
            }

            if($nrestrutlegal || $nrestrutgeren || $lista_nrestrutlegal){
                $condEstrutura = " AND NRVINCULOM IN (SELECT VH.NRVINCULOM
                                 FROM GPE_VINCULOH VH
                                WHERE VH.NRORG = :P_NRORG
                                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||VH.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                                  AND (INSTR(:P_NRESTRUTLEGAL_LIST, '_'||VH.NRESTRUTLEGAL||'_') > 0)
                                  AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
                                  AND (VH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
                                  AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                            WHERE NRORG = VH.NRORG
                                                              AND NRVINCULOM = VH.NRVINCULOM
                                                              AND DTMESCOMPETENC <= :P_DTMESCOMPETENC))";
            }
            
            if(!empty($dtiniadmissao) || !empty($dtfimadmissao)){
                if(!empty($dtiniadmissao) && !empty($dtfimadmissao)){
                    $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM 
                                 FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC >= :P_DTINIADMISSAO AND DTADMISSAOVINC <= :P_DTFIMADMISSAO)";
                } else if(!empty($dtiniadmissao) && empty($dtfimadmissao)){
                    $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC >= :P_DTINIADMISSAO)";
                } else if(empty($dtiniadmissao) && !empty($dtfimadmissao)){
                    $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC <= :P_DTFIMADMISSAO)";
                }
            }
            
            $where = "
                NRORG = :P_NRORG AND
                NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND
                DTMESCOMPETENC = :P_DTMESCOMPETENC AND
                (NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM is null) AND
                (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                AND (NRVINCULOM IN (SELECT NRVINCULOM FROM FPA_ITEMLOTEVINC WHERE NRORG = :P_NRORG AND INSTR(:P_LISTA_LOTE_VINCULOS, '_'||NRLOTEVINC||'_') > 0) OR (:P_LISTA_LOTE_VINCULOS IS NULL OR :P_LISTA_LOTE_VINCULOS = '__'))
                $condTpBeneficio
                $condEstrutura
                $condAdmissao";
            $historicos = VabHistbenefvinc::getObjectsByWhere($where, $params);
            /** @var VabHistbenefvinc $hist */
            foreach ($historicos as $hist) {
                $eventos = array(
                    $hist->getNreventom(),
                    FpaTipoassoceve::ASSOC_LIQUIDO,
                    FpaTipoassoceve::ASSOC_LIQUIDO_PARCIAL
                );
                $params = array(
                    'P_NRORG'           => $nrorg,
                    'P_NRCALCULOFOLHA'  => $nrcalculobenef,
                    'P_DTOCORRENCIA'    => $dtmescompetenc,
                    'P_NRVINCULOM'      => $hist->getNrvinculom(),
                );
                $listEventos = implode(',',$eventos);
                $where = "NRORG = :P_NRORG AND
                          NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND
                          NREVENTOM IN ($listEventos) AND
                          NRVINCULOM = :P_NRVINCULOM";
                $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
                foreach($itemcalcs as $itemcalc){
                    $itemcalc->delete();
                }
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function importaUnimed($row){
        try {
            error_reporting(E_ERROR | E_PARSE);
            $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrorg = $this->hcmParameters->getNrorgTrabalho(); 
            $arquivo = $row['DSPATH'][0];
            $message = array();
            $this->entityManager->beginTransaction();

            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory() . '/');
                   
            if (!$file->isFile()) {
                throw new \Exception("Falha ao ler o arquivo.");
            }
            
            $caminho = $file->getPathname();

            $infos = \PHPExcel_IOFactory::load($caminho);
            $infos->setActiveSheetIndex(0);
            $infos = $this->preparaInfo($infos);
            
            $mescompetenc = explode("/", $dtmescompetenc);
            $mescompetenc = $mescompetenc[2].$mescompetenc[1];
            
            if ($mescompetenc != $infos[0]['COMPETENCIA']){
                throw new \Exception("Por favor altere a competência para a usada no arquivo (".$infos[0]['COMPETENCIA'].")");
            }
            
            $lancamentos = []; //array criado para somar valores de cada beneficiario
            $vinculosCadastrados = [];
            
            foreach ($infos as $info) {
                if(isset($lancamentos[$info['COD_UNIMED'].$info['COD_CONTRATO'].$info['COD_BENEFICIARIO'].$info['COD_DEPENDENTE']]))
                    $lancamentos[$info['COD_UNIMED'].$info['COD_CONTRATO'].$info['COD_BENEFICIARIO'].$info['COD_DEPENDENTE']] += $info['VL_COBRADO'];
                else {
                    $lancamentos[$info['COD_UNIMED'].$info['COD_CONTRATO'].$info['COD_BENEFICIARIO'].$info['COD_DEPENDENTE']] = $info['VL_COBRADO'];
                }
            }

            foreach ($lancamentos as $key => $lancamento) {
                $dadoscop = $this->lancamentoBeneficioRepository->getEventoCoparticipa($nrorg, $dtmescompetenc, $key, $conditions = '');
                
                //validoções
                if (empty($dadoscop)) {
                    $message[]['MENSAGEM'] = "Benefíciario com número de inscrição ".$key." não encontrado!";
                    continue;
                }
                $dadoscop = $dadoscop[0];
                
                if (!isset($dadoscop["NREVENTOCOPART"])) {
                    $message[]['MENSAGEM'] = "Vinculo ".$dadoscop["NMVINCULOM"]." - ".$dadoscop["NRVINCULOM"]." - Evento de coparticipação não cadastrado para Beneficio ".$dadoscop["DSBENEFICIO"];
                    continue;
                }
                
                if(!!$this->existeEvento($nrcalculobenef, $dadoscop["NRVINCULOM"], $nrorg, FpaTipoassoceve::ASSOC_LIQUIDO)){
                    $message[]['MENSAGEM'] = "Vinculo ".$dadoscop["NMVINCULOM"]." - ".$dadoscop["NRVINCULOM"]." com líquido calculado!";
                    continue;
                }
                
                if (!isset($dadoscop["NREVENTOCOPARTDEP"])) 
                    $dadoscop["NREVENTOCOPARTDEP"] = $dadoscop["NREVENTOCOPART"];
                    
                    
                if($dadoscop["IDTPBENEFICIARIO"] == 'T'){
                    $eventoEmp = $dadoscop["NREVETITULAREMP"];
                    $eventoCop = $dadoscop["NREVENTOCOPART"];
                    $percCop = $dadoscop["PERCOPARTICIPATITU"];
                }  else{
                    $eventoEmp = $dadoscop["NREVEDEPEMPRESA"];
                    $eventoCop = $dadoscop["NREVENTOCOPARTDEP"];
                    $percCop = $dadoscop["PERCOPARTICIPADEPE"];
                }
                
                
                $valorBeneficiario = ($lancamento*(100-$percCop))/100;
                $valorEmpresa = $lancamento - $valorBeneficiario;
                
                //insere no item de calculo 
                if(!!$this->existeEvento($nrcalculobenef, $dadoscop["NRVINCULOM"], $nrorg, $eventoCop)){
                    $itemcalc = $this->existeEvento($nrcalculobenef, $dadoscop["NRVINCULOM"], $nrorg, $eventoCop);
                    $itemcalc->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $itemcalc->setDtultatu(DateUtil::getDataAtual());
                    $itemcalc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    
                    
                    $itemcalcEmpr = $this->existeEventoCopEmpresa($nrcalculobenef, $dadoscop["NRVINCULOM"], $nrorg, $eventoEmp, $dtmescompetenc);
                    
                    if(!$itemcalcEmpr) {
                        $itemcalcEmpr = clone $itemcalc;
                        $itemcalcEmpr->setVreventofolha(0);
                        $itemcalcEmpr->setNreventom($eventoEmp);
                        $itemcalcEmpr->setDtocorrencia(DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true)));    
                        $itemcalcEmpr->setNritemcalcfolha(NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 21, true));
                    }
                    
                    
                    
                    if($row["IDSOBRESCREVE"] == 'N' || in_array($dadoscop["NRVINCULOM"],$vinculosCadastrados)){
                        $valorBeneficiario += $itemcalc->getVreventofolha(); 
                        $valorEmpresa += $itemcalcEmpr->getVreventofolha();
                    } 
                    
                    $itemcalcEmpr->setVreventofolha($valorEmpresa);
                    $itemcalc->setVreventofolha($valorBeneficiario);
                    
                    
                } else {
                    $ideventodemonst = FpaEventoh::getIdeventodemonst($eventoCop, $dtmescompetenc);
                    
                    $itemcalc = new FpaItecalcfolha();
                    $itemcalc->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $itemcalc->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $itemcalc->setDtinclusao(DateUtil::getDataAtual());
                    $itemcalc->setDtultatu(DateUtil::getDataAtual());
                    $itemcalc->setIdevendemoncalc($ideventodemonst);
                    $itemcalc->setIdrefevalorcalc('NORMAL');
                    $itemcalc->setNrcalculofolha($nrcalculobenef);
                    $itemcalc->setNreventom($eventoCop);
                    $itemcalc->setNrorg($this->hcmParameters->getNrorgTrabalho());
                    $itemcalc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                    $itemcalc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $itemcalc->setNrvinculom($dadoscop["NRVINCULOM"]);
                    
                    $itemcalc->setQtocorproxperi(1);
                    $itemcalc->setVreventofolha($valorBeneficiario);
                    $itemcalc->setDtocorrencia(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true));
                    $itemcalc->setNritemcalcfolha(NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 21, true));
                    
                    $itemcalcEmpr = clone $itemcalc;
                    $itemcalcEmpr->setVreventofolha($valorEmpresa);
                    $itemcalcEmpr->setNreventom($eventoEmp);
                    $itemcalcEmpr->setDtocorrencia(DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true)));
                    $itemcalcEmpr->setNritemcalcfolha(NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 21, true));
                    
                }

                $this->entityManager->persist($itemcalc);
				$this->entityManager->flush($itemcalc);
                $this->entityManager->persist($itemcalcEmpr);
				$this->entityManager->flush($itemcalcEmpr);
                
                
                
                $vinculosCadastrados[] = $dadoscop["NRVINCULOM"];
                
            }
            
            $this->entityManager->commit();
            return $message;
        
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    protected function preparaInfo( $info) {
        $idLinha = 9;
        $ColCompetec = 'A';
        $ColContrato = 'B';
        $ColCodUnimed = 'C';
        $ColCodCntrat = 'D';
        $ColCodBnf = 'E';
        $ColCodDepnte = 'F';
        $ColNmBeneficiario = 'G';
        $ColNrMatr = 'H';
        $ColIdGuia = 'I';
        $ColDtAtend = 'J';
        $ColIdDesc = 'K';
        $ColNrQtde = 'L';
        $ColVlBase = 'M';
        $ColVlCob = 'N';
  
        $infoFinal = array();
        
        while(!empty($info->getActiveSheet()->getCell($ColContrato.$idLinha)->getValue())){
            
            $reg = array();
            $reg['COMPETENCIA'] = $info->getActiveSheet()->getCell($ColCompetec.$idLinha)->getValue();
            $reg['NR_CONTRATO'] = $info->getActiveSheet()->getCell($ColContrato.$idLinha)->getValue();
            $reg['COD_UNIMED'] = $info->getActiveSheet()->getCell($ColCodUnimed.$idLinha)->getValue();
            $reg['COD_CONTRATO'] = $info->getActiveSheet()->getCell($ColCodCntrat.$idLinha)->getValue();
            $reg['COD_BENEFICIARIO'] = $info->getActiveSheet()->getCell($ColCodBnf.$idLinha)->getValue();
            $reg['COD_DEPENDENTE'] = $info->getActiveSheet()->getCell($ColCodDepnte.$idLinha)->getValue();
            $reg['NM_BENEFICIARIO'] = $info->getActiveSheet()->getCell($ColNmBeneficiario.$idLinha)->getValue();
            $reg['NR_MATRICULA'] = $info->getActiveSheet()->getCell($ColNrMatr.$idLinha)->getValue();
            $reg['ID_GUIA'] = $info->getActiveSheet()->getCell($ColIdGuia.$idLinha)->getValue();
            $reg['DT_ATENDIMENTO'] = $info->getActiveSheet()->getCell($ColDtAtend.$idLinha)->getValue();
            $reg['DESCRICAO'] = $info->getActiveSheet()->getCell($ColIdDesc.$idLinha)->getValue();
            $reg['QUANTIDADE'] = $info->getActiveSheet()->getCell($ColNrQtde.$idLinha)->getValue();
            $reg['VL_BASE'] = $info->getActiveSheet()->getCell($ColVlBase.$idLinha)->getValue();
            $reg['VL_COBRADO'] = $info->getActiveSheet()->getCell($ColVlCob.$idLinha)->getValue();


            $idLinha++;

            $infoFinal[] = $reg;
        }
        

        return $infoFinal;
    }

}